# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)



DESCRIPTOR = descriptor.FileDescriptor(
  name='test.proto',
  package='test',
  serialized_pb='\n\ntest.proto\x12\x04test\"\x1f\n\x07SubTest\x12\t\n\x01\x61\x18\x01 \x01(\x05\x12\t\n\x01\x62\x18\x02 \x01(\x11\":\n\x0cRecursiveFoo\x12\x1f\n\x03\x62\x61r\x18\x01 \x01(\x0b\x32\x12.test.RecursiveBar\x12\t\n\x01n\x18\x02 \x01(\x05\"/\n\x0cRecursiveBar\x12\x1f\n\x03\x66oo\x18\x01 \x02(\x0b\x32\x12.test.RecursiveFoo\"1\n\x07SelfRef\x12\x1b\n\x04self\x18\x01 \x01(\x0b\x32\r.test.SelfRef\x12\t\n\x01n\x18\x02 \x01(\x05\"\xc7\x02\n\x04Test\x12\t\n\x01\x61\x18\x01 \x01(\x05\x12\t\n\x01\x62\x18\x02 \x01(\x03\x12\t\n\x01\x63\x18\x03 \x01(\x11\x12\t\n\x01\x64\x18\x04 \x01(\x12\x12\t\n\x01\x65\x18\x05 \x01(\x07\x12\t\n\x01\x66\x18\x06 \x01(\x06\x12\t\n\x01g\x18\x07 \x01(\x0f\x12\t\n\x01h\x18\x08 \x01(\x10\x12\t\n\x01i\x18\t \x01(\x02\x12\t\n\x01j\x18\n \x01(\x01\x12\t\n\x01k\x18\x0b \x01(\r\x12\t\n\x01l\x18\x0c \x01(\x04\x12\t\n\x01m\x18\r \x01(\t\x12\t\n\x01n\x18\x0e \x01(\x08\x12\x18\n\x01o\x18\x0f \x01(\x0b\x32\r.test.SubTest\x12\t\n\x01p\x18\x10 \x03(\x05\x12\r\n\x01q\x18\x11 \x03(\x05\x42\x02\x10\x01\x12\x18\n\x01r\x18\x12 \x03(\x0b\x32\r.test.SubTest\x12\x19\n\x01s\x18\x13 \x01(\x0e\x32\x0e.test.TestType\x12\x1f\n\x03\x66oo\x18\x14 \x01(\x0b\x32\x12.test.RecursiveFoo\x12\x1b\n\x04self\x18\x15 \x01(\x0b\x32\r.test.SelfRef*4\n\x08TestType\x12\x12\n\x05TYPE1\x10\xff\xff\xff\xff\xff\xff\xff\xff\xff\x01\x12\t\n\x05TYPE2\x10\x02\x12\t\n\x05TYPE3\x10\x03')

_TESTTYPE = descriptor.EnumDescriptor(
  name='TestType',
  full_name='test.TestType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    descriptor.EnumValueDescriptor(
      name='TYPE1', index=0, number=-1,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='TYPE2', index=1, number=2,
      options=None,
      type=None),
    descriptor.EnumValueDescriptor(
      name='TYPE3', index=2, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=543,
  serialized_end=595,
)


TYPE1 = -1
TYPE2 = 2
TYPE3 = 3



_SUBTEST = descriptor.Descriptor(
  name='SubTest',
  full_name='test.SubTest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='a', full_name='test.SubTest.a', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='b', full_name='test.SubTest.b', index=1,
      number=2, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=20,
  serialized_end=51,
)


_RECURSIVEFOO = descriptor.Descriptor(
  name='RecursiveFoo',
  full_name='test.RecursiveFoo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='bar', full_name='test.RecursiveFoo.bar', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='n', full_name='test.RecursiveFoo.n', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=53,
  serialized_end=111,
)


_RECURSIVEBAR = descriptor.Descriptor(
  name='RecursiveBar',
  full_name='test.RecursiveBar',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='foo', full_name='test.RecursiveBar.foo', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=113,
  serialized_end=160,
)


_SELFREF = descriptor.Descriptor(
  name='SelfRef',
  full_name='test.SelfRef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='self', full_name='test.SelfRef.self', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='n', full_name='test.SelfRef.n', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=162,
  serialized_end=211,
)


_TEST = descriptor.Descriptor(
  name='Test',
  full_name='test.Test',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='a', full_name='test.Test.a', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='b', full_name='test.Test.b', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='c', full_name='test.Test.c', index=2,
      number=3, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='d', full_name='test.Test.d', index=3,
      number=4, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='e', full_name='test.Test.e', index=4,
      number=5, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='f', full_name='test.Test.f', index=5,
      number=6, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='g', full_name='test.Test.g', index=6,
      number=7, type=15, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='h', full_name='test.Test.h', index=7,
      number=8, type=16, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='i', full_name='test.Test.i', index=8,
      number=9, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='j', full_name='test.Test.j', index=9,
      number=10, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='k', full_name='test.Test.k', index=10,
      number=11, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='l', full_name='test.Test.l', index=11,
      number=12, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='m', full_name='test.Test.m', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='n', full_name='test.Test.n', index=13,
      number=14, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='o', full_name='test.Test.o', index=14,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='p', full_name='test.Test.p', index=15,
      number=16, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='q', full_name='test.Test.q', index=16,
      number=17, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=descriptor._ParseOptions(descriptor_pb2.FieldOptions(), '\020\001')),
    descriptor.FieldDescriptor(
      name='r', full_name='test.Test.r', index=17,
      number=18, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='s', full_name='test.Test.s', index=18,
      number=19, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=-1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='foo', full_name='test.Test.foo', index=19,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='self', full_name='test.Test.self', index=20,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=214,
  serialized_end=541,
)

_RECURSIVEFOO.fields_by_name['bar'].message_type = _RECURSIVEBAR
_RECURSIVEBAR.fields_by_name['foo'].message_type = _RECURSIVEFOO
_SELFREF.fields_by_name['self'].message_type = _SELFREF
_TEST.fields_by_name['o'].message_type = _SUBTEST
_TEST.fields_by_name['r'].message_type = _SUBTEST
_TEST.fields_by_name['s'].enum_type = _TESTTYPE
_TEST.fields_by_name['foo'].message_type = _RECURSIVEFOO
_TEST.fields_by_name['self'].message_type = _SELFREF
DESCRIPTOR.message_types_by_name['SubTest'] = _SUBTEST
DESCRIPTOR.message_types_by_name['RecursiveFoo'] = _RECURSIVEFOO
DESCRIPTOR.message_types_by_name['RecursiveBar'] = _RECURSIVEBAR
DESCRIPTOR.message_types_by_name['SelfRef'] = _SELFREF
DESCRIPTOR.message_types_by_name['Test'] = _TEST

class SubTest(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SUBTEST
  
  # @@protoc_insertion_point(class_scope:test.SubTest)

class RecursiveFoo(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _RECURSIVEFOO
  
  # @@protoc_insertion_point(class_scope:test.RecursiveFoo)

class RecursiveBar(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _RECURSIVEBAR
  
  # @@protoc_insertion_point(class_scope:test.RecursiveBar)

class SelfRef(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _SELFREF
  
  # @@protoc_insertion_point(class_scope:test.SelfRef)

class Test(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TEST
  
  # @@protoc_insertion_point(class_scope:test.Test)

# @@protoc_insertion_point(module_scope)



# coding: utf-8
from cprotobuf import ProtoEntity, Field
# file: descriptor.proto
class SourceCodeInfo(ProtoEntity):
    location        = Field('SourceCodeInfo.Location',	1, repeated=True)

class UninterpretedOption(ProtoEntity):
    name            = Field('UninterpretedOption.NamePart',	2, repeated=True)
    identifier_value = Field('string',	3, required=False)
    positive_int_value = Field('uint64',	4, required=False)
    negative_int_value = Field('int64',	5, required=False)
    double_value    = Field('double',	6, required=False)
    string_value    = Field('bytes',	7, required=False)
    aggregate_value = Field('string',	8, required=False)

class EnumValueOptions(ProtoEntity):
    deprecated      = Field('bool',	1, required=False, default=False)
    uninterpreted_option = Field(UninterpretedOption,	999, repeated=True)

class FieldOptions(ProtoEntity):
    # enum CType
    STRING=0
    CORD=1
    STRING_PIECE=2
    # enum JSType
    JS_NORMAL=0
    JS_STRING=1
    JS_NUMBER=2
    ctype           = Field('enum',	1, required=False, default=STRING)
    packed          = Field('bool',	2, required=False)
    jstype          = Field('enum',	6, required=False, default=JS_NORMAL)
    lazy            = Field('bool',	5, required=False, default=False)
    deprecated      = Field('bool',	3, required=False, default=False)
    weak            = Field('bool',	10, required=False, default=False)
    uninterpreted_option = Field(UninterpretedOption,	999, repeated=True)

class MessageOptions(ProtoEntity):
    message_set_wire_format = Field('bool',	1, required=False, default=False)
    no_standard_descriptor_accessor = Field('bool',	2, required=False, default=False)
    deprecated      = Field('bool',	3, required=False, default=False)
    map_entry       = Field('bool',	7, required=False)
    uninterpreted_option = Field(UninterpretedOption,	999, repeated=True)

class ExtensionRange(ProtoEntity):
    start           = Field('int32',	1, required=False)
    end             = Field('int32',	2, required=False)

class EnumValueDescriptorProto(ProtoEntity):
    name            = Field('string',	1, required=False)
    number          = Field('int32',	2, required=False)
    options         = Field(EnumValueOptions,	3, required=False)

class FileOptions(ProtoEntity):
    # enum OptimizeMode
    SPEED=1
    CODE_SIZE=2
    LITE_RUNTIME=3
    java_package    = Field('string',	1, required=False)
    java_outer_classname = Field('string',	8, required=False)
    java_multiple_files = Field('bool',	10, required=False, default=False)
    java_generate_equals_and_hash = Field('bool',	20, required=False, default=False)
    java_string_check_utf8 = Field('bool',	27, required=False, default=False)
    optimize_for    = Field('enum',	9, required=False, default=SPEED)
    go_package      = Field('string',	11, required=False)
    cc_generic_services = Field('bool',	16, required=False, default=False)
    java_generic_services = Field('bool',	17, required=False, default=False)
    py_generic_services = Field('bool',	18, required=False, default=False)
    deprecated      = Field('bool',	23, required=False, default=False)
    cc_enable_arenas = Field('bool',	31, required=False, default=False)
    objc_class_prefix = Field('string',	36, required=False)
    csharp_namespace = Field('string',	37, required=False)
    javanano_use_deprecated_package = Field('bool',	38, required=False)
    uninterpreted_option = Field(UninterpretedOption,	999, repeated=True)

class ServiceOptions(ProtoEntity):
    deprecated      = Field('bool',	33, required=False, default=False)
    uninterpreted_option = Field(UninterpretedOption,	999, repeated=True)

class OneofDescriptorProto(ProtoEntity):
    name            = Field('string',	1, required=False)

class FieldDescriptorProto(ProtoEntity):
    # enum Type
    TYPE_DOUBLE=1
    TYPE_FLOAT=2
    TYPE_INT64=3
    TYPE_UINT64=4
    TYPE_INT32=5
    TYPE_FIXED64=6
    TYPE_FIXED32=7
    TYPE_BOOL=8
    TYPE_STRING=9
    TYPE_GROUP=10
    TYPE_MESSAGE=11
    TYPE_BYTES=12
    TYPE_UINT32=13
    TYPE_ENUM=14
    TYPE_SFIXED32=15
    TYPE_SFIXED64=16
    TYPE_SINT32=17
    TYPE_SINT64=18
    # enum Label
    LABEL_OPTIONAL=1
    LABEL_REQUIRED=2
    LABEL_REPEATED=3
    name            = Field('string',	1, required=False)
    number          = Field('int32',	3, required=False)
    label           = Field('enum',	4, required=False)
    type            = Field('enum',	5, required=False)
    type_name       = Field('string',	6, required=False)
    extendee        = Field('string',	2, required=False)
    default_value   = Field('string',	7, required=False)
    oneof_index     = Field('int32',	9, required=False)
    json_name       = Field('string',	10, required=False)
    options         = Field(FieldOptions,	8, required=False)

class EnumOptions(ProtoEntity):
    allow_alias     = Field('bool',	2, required=False)
    deprecated      = Field('bool',	3, required=False, default=False)
    uninterpreted_option = Field(UninterpretedOption,	999, repeated=True)

class EnumDescriptorProto(ProtoEntity):
    name            = Field('string',	1, required=False)
    value           = Field(EnumValueDescriptorProto,	2, repeated=True)
    options         = Field(EnumOptions,	3, required=False)

class DescriptorProto(ProtoEntity):
    name            = Field('string',	1, required=False)
    field           = Field(FieldDescriptorProto,	2, repeated=True)
    extension       = Field(FieldDescriptorProto,	6, repeated=True)
    nested_type     = Field('DescriptorProto',	3, repeated=True)
    enum_type       = Field(EnumDescriptorProto,	4, repeated=True)
    extension_range = Field(ExtensionRange,	5, repeated=True)
    oneof_decl      = Field(OneofDescriptorProto,	8, repeated=True)
    options         = Field(MessageOptions,	7, required=False)
    reserved_range  = Field('DescriptorProto.ReservedRange',	9, repeated=True)
    reserved_name   = Field('string',	10, repeated=True)

class GeneratedCodeInfo(ProtoEntity):
    annotation      = Field('GeneratedCodeInfo.Annotation',	1, repeated=True)

class MethodOptions(ProtoEntity):
    deprecated      = Field('bool',	33, required=False, default=False)
    uninterpreted_option = Field(UninterpretedOption,	999, repeated=True)

class MethodDescriptorProto(ProtoEntity):
    name            = Field('string',	1, required=False)
    input_type      = Field('string',	2, required=False)
    output_type     = Field('string',	3, required=False)
    options         = Field(MethodOptions,	4, required=False)
    client_streaming = Field('bool',	5, required=False, default=False)
    server_streaming = Field('bool',	6, required=False, default=False)

class ServiceDescriptorProto(ProtoEntity):
    name            = Field('string',	1, required=False)
    method          = Field(MethodDescriptorProto,	2, repeated=True)
    options         = Field(ServiceOptions,	3, required=False)

class FileDescriptorProto(ProtoEntity):
    name            = Field('string',	1, required=False)
    package         = Field('string',	2, required=False)
    dependency      = Field('string',	3, repeated=True)
    public_dependency = Field('int32',	10, repeated=True)
    weak_dependency = Field('int32',	11, repeated=True)
    message_type    = Field(DescriptorProto,	4, repeated=True)
    enum_type       = Field(EnumDescriptorProto,	5, repeated=True)
    service         = Field(ServiceDescriptorProto,	6, repeated=True)
    extension       = Field(FieldDescriptorProto,	7, repeated=True)
    options         = Field(FileOptions,	8, required=False)
    source_code_info = Field(SourceCodeInfo,	9, required=False)
    syntax          = Field('string',	12, required=False)

class FileDescriptorSet(ProtoEntity):
    file            = Field(FileDescriptorProto,	1, repeated=True)

# file: plugin.proto
class CodeGeneratorRequest(ProtoEntity):
    file_to_generate = Field('string',	1, repeated=True)
    parameter       = Field('string',	2, required=False)
    proto_file      = Field(FileDescriptorProto,	15, repeated=True)

class CodeGeneratorResponse_File(ProtoEntity):
    name            = Field('string',	1, required=False)
    insertion_point = Field('string',	2, required=False)
    content         = Field('string',	15, required=False)

class CodeGeneratorResponse(ProtoEntity):
    error           = Field('string',	1, required=False)
    file            = Field(CodeGeneratorResponse_File,	15, repeated=True)




#!/usr/bin/env python
from distutils.core import setup
from distutils.extension import Extension

try:
    from Cython.Distutils import build_ext
    have_cython = True
except ImportError:
    have_cython = False

#if have_cython:
if False:
    ext_modules = [Extension("cprotobuf.internal", ["cprotobuf/internal.pyx"])
                   ]
    cmdclass = {'build_ext': build_ext}
else:
    cmdclass = {}
    ext_modules = [Extension("cprotobuf.internal", ["cprotobuf/internal.c"])
                   ]

setup(
    version='0.1.9',
    name='cprotobuf',
    ext_modules=ext_modules,
    scripts=['protoc-gen-cprotobuf'],
    packages=['cprotobuf'],
    author='huangyi',
    author_email='yi.codeplayer@gmail.com',
    url='https://github.com/yihuang/cprotobuf',
    description='pythonic and high performance protocol buffer implementation.',
    cmdclass=cmdclass,
    long_description=open('README.rst').read(),
)



from cpython cimport *
from libc.stdint cimport *

# {{{ definitions

cdef extern from "Python.h":
    Py_ssize_t PyByteArray_GET_SIZE(object array)
    object PyUnicode_FromStringAndSize(char *buff, Py_ssize_t len)
    int PyByteArray_Resize(object self, Py_ssize_t size) except -1
    char* PyByteArray_AS_STRING(object bytearray)

ctypedef object(*Decoder)(char **pointer, char *end)
ctypedef object(*Encoder)(Field _f, bytearray array, object value)

class InternalDecodeError(Exception):
    pass

cdef inline object makeDecodeError(char* pointer, message):
    cdef uint64_t locator = <uint64_t>pointer
    return InternalDecodeError(locator, message)

class DecodeError(Exception):
    def __init__(self, pointer, message):
        self.pointer = pointer
        self.message = message
    def __str__(self):
        return self.message.format(self.pointer)

# }}}

# {{{ decoding

# {{{ raw stuff

cdef inline int raw_decode_uint32(char **start, char *end, uint32_t *result) nogil:
    cdef uint32_t value = 0
    cdef uint32_t byte
    cdef char *pointer = start[0]
    cdef int counter = 0
    while True:
        if pointer >= end:
            return -1
        byte = pointer[0]
        value |= (byte & 0x7f) << counter
        counter+=7
        pointer+=1
        if byte & 0x80 == 0:
            break
    start[0] = pointer
    result[0] = value
    return 0

cdef inline int raw_decode_uint64(char **start, char *end, uint64_t *result) nogil:
    cdef uint64_t value = 0
    cdef uint64_t byte
    cdef char *pointer = start[0]
    cdef int counter = 0

    while True:
        if pointer >= end:
            return -1
        byte = pointer[0]
        value |= (byte & 0x7f) << counter
        counter+=7
        pointer+=1
        if byte & 0x80 == 0:
            break
    start[0] = pointer
    result[0] = value
    return 0

cdef inline int raw_decode_fixed32(char **pointer, char *end, uint32_t *result) nogil:
    cdef uint32_t value = 0
    cdef char *start = pointer[0]
    cdef int i

    for i from 0 <= i < 4:
        if start == end:
            return -1
        value |= <unsigned char>start[0] << (i * 8)
        start += 1
    pointer[0] = start
    result[0] = value
    return 0

cdef inline int raw_decode_fixed64(char **pointer, char *end, uint64_t *result) nogil:
    cdef uint64_t value = 0
    cdef char *start = pointer[0]
    cdef uint64_t temp = 0
    cdef int i
    for i from 0 <= i < 8:
        if start == end:
            return -1
        temp = <unsigned char>start[0]
        value |= temp << (i * 8)
        start += 1
    pointer[0] = start
    result[0] = value
    return 0

cdef inline int raw_decode_delimited(char **pointer, char *end, char **result, uint64_t *size) nogil:
    if raw_decode_uint64(pointer, end, size):
        return -1

    cdef char* start = pointer[0]
    if start+size[0] > end:
        return -2

    result[0] = start
    pointer[0] = start+size[0]
    return 0

cdef inline int skip_unknown_field(char **pointer, char *end, int wtype) nogil:
    cdef uint32_t size
    cdef char* start
    if wtype == 0:
        start = pointer[0]
        while True:
            if start >= end:
                return -1
            if start[0] & 0x80 == 0:
                break
            start += 1
        pointer[0] = start + 1
    elif wtype == 1:
        pointer[0] += 8
    elif wtype == 2:
        if raw_decode_uint32(pointer, end, &size):
            return -1
        if pointer[0]+size > end:
            return -1
        pointer[0] += size
    elif wtype == 5:
        pointer[0] += 4
    else:
        return -1
    return 0

# }}}

cdef object decode_uint32(char **pointer, char *end):
    cdef uint32_t result
    if raw_decode_uint32(pointer, end, &result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `uint32` at [{0}]")

    return result

cdef object decode_uint64(char **pointer, char *end):
    cdef uint64_t result
    if raw_decode_uint64(pointer, end, &result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `uint64` at [{0}]")

    return result

cdef object decode_int32(char **pointer, char *end, ):
    cdef int32_t result
    if raw_decode_uint32(pointer, end, <uint32_t*>&result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `int32` at [{0}]")

    return result

cdef object decode_int64(char **pointer, char *end, ):
    cdef int64_t result
    if raw_decode_uint64(pointer, end, <uint64_t*>&result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `int64` at [{0}]")

    return result

cdef object decode_sint32(char **pointer, char *end, ):
    cdef uint32_t result
    if raw_decode_uint32(pointer, end, &result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `sint32` at [{0}]")

    return <int32_t>((result >> 1) ^ (-<int32_t>(result & 1)))

cdef object decode_sint64(char **pointer, char *end, ):
    cdef uint64_t un
    if raw_decode_uint64(pointer, end, &un):
        raise makeDecodeError(pointer[0], "Can't decode value of type `sint64` at [{0}]")

    return <int64_t>((un >> 1) ^ (-<int64_t>(un & 1)))

cdef object decode_fixed32(char **pointer, char *end, ):
    cdef uint32_t result
    if raw_decode_fixed32(pointer, end, &result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `fixed32` at [{0}]")

    return result

cdef object decode_fixed64(char **pointer, char *end, ):
    cdef uint64_t result
    if raw_decode_fixed64(pointer, end, &result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `fixed64` at [{0}]")

    return result

cdef object decode_sfixed32(char **pointer, char *end, ):
    cdef int32_t result
    if raw_decode_fixed32(pointer, end, <uint32_t*>&result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed32` at [{0}]")

    return result

cdef object decode_sfixed64(char **pointer, char *end, ):
    cdef int64_t result
    if raw_decode_fixed64(pointer, end, <uint64_t*>&result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `sfixed64` at [{0}]")

    return result

cdef object decode_bytes(char **pointer, char *end, ):
    cdef char *result
    cdef uint64_t size
    cdef int ret = raw_decode_delimited(pointer, end, &result, &size)
    if ret==0:
        return PyBytes_FromStringAndSize(result, size)

    if ret == -1:
        raise makeDecodeError(pointer[0], "Can't decode size for value of type `bytes` at [{0}]")
    elif ret == -2:
        raise makeDecodeError(pointer[0], "Can't decode value of type `bytes` of size %d at [{0}]" % size)


cdef object decode_string(char **pointer, char *end, ):
    cdef char *result
    cdef uint64_t size
    cdef int ret = raw_decode_delimited(pointer, end, &result, &size)
    if ret==0:
        return PyUnicode_FromStringAndSize(result, size)

    if ret == -1:
        raise makeDecodeError(pointer[0], "Can't decode size for value of type `string` at [{0}]")
    elif ret == -2:
        raise makeDecodeError(pointer[0], "Can't decode value of type `string` of size %d at [{0}]" % size)

cdef object decode_float(char **pointer, char *end, ):
    cdef float result
    if raw_decode_fixed32(pointer, end, <uint32_t*>&result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `float` at [{0}]")

    return result

cdef object decode_double(char **pointer, char *end, ):
    cdef double result
    if raw_decode_fixed64(pointer, end, <uint64_t*>&result):
        raise makeDecodeError(pointer[0], "Can't decode value of type `double` at [{0}]")

    return result

cdef object decode_bool(char **pointer, char *end, ):
    cdef char* start = pointer[0]
    pointer[0] = start + 1

    return <bint>start[0]

# }}}

# {{{ encoding

cdef inline int raw_encode_uint32(bytearray array, uint32_t n) except -1:
    cdef unsigned short int rem
    cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
    PyByteArray_Resize(array, size + 10)
    cdef char *buff = PyByteArray_AS_STRING(array) + size

    if 0!=n:
        while True:
            rem = <char>(n & 0x7f)
            n = n>>7
            if 0==n:
                buff[0] = <char> rem
                buff+=1
                break
            else:
                rem = rem | 0x80
                buff[0] = <char> rem
                buff+=1
    else:
        buff[0] = '\0'
        buff+=1

    PyByteArray_Resize(array, buff - PyByteArray_AS_STRING(array))
    return 0

cdef inline encode_uint32(Field _f, bytearray array, object value):
    raw_encode_uint32(array, value)

cdef inline encode_int32(Field _f, bytearray array, object value):
    cdef int32_t n = value
    raw_encode_uint32(array, <uint32_t>n)

cdef inline encode_sint32(Field _f, bytearray array, object value):
    cdef int32_t n = value
    cdef uint32_t un = (n << 1) ^ (n >> 31)

    raw_encode_uint32(array, un)

cdef inline int raw_encode_uint64(bytearray array, uint64_t n) except -1:
    cdef unsigned short int rem
    cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
    PyByteArray_Resize(array, size + 20)
    cdef char *buff = PyByteArray_AS_STRING(array) + size

    if 0!=n:
        while True:
            rem = <char>(n & 0x7f)
            n = n>>7
            if 0==n:
                buff[0] = <char> rem
                buff+=1
                break
            else:
                rem = rem | 0x80
                buff[0] = <char> rem
                buff+=1
    else:
        buff[0] = '\0'
        buff+=1
    PyByteArray_Resize(array, buff - PyByteArray_AS_STRING(array))
    return 0

cdef inline encode_uint64(Field _f, bytearray array, object value):
    raw_encode_uint64(array, value)

cdef inline encode_int64(Field _f, bytearray array, object value):
    cdef int64_t n = value
    raw_encode_uint64(array, <uint64_t>n)

cdef inline encode_sint64(Field _f, bytearray array, object value):
    cdef int64_t n = value
    cdef uint64_t un = (n<<1) ^ (n>>63)
    raw_encode_uint64(array, un)

cdef inline int raw_encode_fixed32(bytearray array, uint32_t n) except -1:
    cdef unsigned short int rem
    cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
    PyByteArray_Resize(array, size + 4)
    cdef char *buff = PyByteArray_AS_STRING(array) + size
    cdef int i

    for i from 0 <= i < 4:
        rem = n & 0xff
        n = n >> 8
        buff[0] = <char> rem
        buff += 1

    return 0

cdef inline encode_fixed32(Field _f, bytearray array, object value):
    raw_encode_fixed32(array, value)

cdef inline encode_sfixed32(Field _f, bytearray array, object value):
    cdef int32_t n = value
    cdef unsigned short int rem
    cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
    PyByteArray_Resize(array, size + 4)
    cdef char *buff = PyByteArray_AS_STRING(array) + size
    cdef int i

    for i from 0 <= i < 4:
        rem = n & 0xff
        n = n >> 8
        buff[0] = <char> rem
        buff += 1

cdef inline int raw_encode_fixed64(bytearray array, uint64_t n) except -1:
    cdef unsigned short int rem
    cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
    PyByteArray_Resize(array, size + 8)
    cdef char *buff = PyByteArray_AS_STRING(array) + size
    cdef int i

    for i from 0 <= i < 8:
        rem = n & 0xff
        n = n >> 8
        buff[0] = <char> rem
        buff += 1

    return 0

cdef inline encode_fixed64(Field _f, bytearray array, object value):
    raw_encode_fixed64(array, value)

cdef inline encode_sfixed64(Field _f, bytearray array, object value):
    cdef int64_t n = value
    cdef unsigned short int rem
    cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
    PyByteArray_Resize(array, size + 8)
    cdef char *buff = PyByteArray_AS_STRING(array) + size
    cdef int i

    for i from 0 <= i < 8:
        rem = n & 0xff
        n = n >> 8
        buff[0] = <char> rem
        buff += 1

cdef inline encode_bytes(Field _f, bytearray array, object n):
    cdef Py_ssize_t len = PySequence_Length(n)
    raw_encode_uint64(array, len)
    PySequence_InPlaceConcat(array, n)

cdef inline encode_string(Field _f, bytearray array, object n):
    if isinstance(n, unicode):
        n = PyUnicode_AsUTF8String(n)
    cdef Py_ssize_t len = PySequence_Length(n)
    raw_encode_uint64(array, len)
    PySequence_InPlaceConcat(array, n)

cdef inline encode_bool(Field _f, bytearray array, object value):
    cdef bint b = value
    cdef Py_ssize_t size = PyByteArray_GET_SIZE(array)
    PyByteArray_Resize(array, size + 1)
    cdef char *buff = PyByteArray_AS_STRING(array) + size
    buff[0] = b

cdef inline encode_float(Field _f, bytearray array, object value):
    cdef float f = value
    raw_encode_fixed32(array, (<uint32_t*>&f)[0])

cdef inline encode_double(Field _f, bytearray array, object value):
    cdef double d = value
    raw_encode_fixed64(array, (<uint64_t*>&d)[0])

cdef inline encode_type(bytearray array, unsigned char wire_type, uint32_t index):
    raw_encode_uint32(array, index<<3|wire_type)

# }}}



# coding: utf-8
import itertools
import pyximport; pyximport.install()
from cprotobuf import ProtoEntity, Field, encode_data
import test_pb2
import test_pb

data1 = dict(
    a = 2147483647,
    b = 9223372036854775807,
    c = 2147483647,
    d = 9223372036854775807,
    e = 4294967295,
    f = 18446744073709551615,
    g = 2147483647,
    h = 9223372036854775807,
    i = 0.3,
    j = 0.3,
    k = 4294967295,
    l = 18446744073709551615,
    m = u'测试',
    n = True,
    o = dict(
        a = 150,
        b = -150,
    ),
    p = [1,2,3],
    q = [1,2,3],
    r = [
        dict(
            a = 150,
            b = -150
        ),
    ],
    s = test_pb2.TYPE1,
    foo = {
        'bar': {
            'foo': {
                'bar': {
                    'foo': {
                        'n': 1,
                    },
                }
            },
        }
    },
    self = {
        'self': {
            'n': 1,
        },
    }
)

data2 = dict(
    a = -2147483647,
    b = -9223372036854775807,
    c = -2147483647,
    d = -9223372036854775807,
    e = 2147483647,
    f = 9223372036854775807,
    g = -2147483647,
    h = -9223372036854775807,
    i = -0.3,
    j = -0.3,
    k = 4294967295,
    l = 18446744073709551615,
    m = u'测试',
    n = False,
    s = test_pb2.TYPE2,
)

def set_obj(o, d):
    for k,v in d.items():
        if isinstance(v, dict):
            set_obj(getattr(o, k), v)
        elif isinstance(v, list):
            for i in v:
                attr = getattr(o, k)
                if isinstance(i, dict):
                    set_obj(attr.add(), i)
                else:
                    attr.append(i)
        else:
            setattr(o, k, v)

def eq_obj(data, obj1, obj2):
    for f, v in data.items():
        v1 = getattr(obj1, f)
        v2 = getattr(obj2, f)
        if isinstance(v, list):
            for a, b in itertools.izip_longest(v1, v2):
                if isinstance(v[0], dict):
                    eq_obj(v[0], a, b)
                else:
                    assert a==b, (f, a, b)
        elif isinstance(v, dict):
            eq_obj(v, v1, v2)
        else:
            assert v1==v2, (f, v1, v2)

def _test(data):
    e_obj1 = test_pb2.Test()
    set_obj(e_obj1, data)
    e_obj2 = test_pb.Test()
    set_obj(e_obj2, data)

    bs1 = e_obj1.SerializeToString()
    bs2 = str(e_obj2.SerializeToString())

    bs3 = bytearray()
    encode_data(bs3, test_pb.Test, data)

    if bs1 != bs2:
        print(len(bs1), repr(bs1))
        print(len(bs2), repr(bs2))
        assert False, 'encoding result is not the same'

    assert bs1 == bs2 == bs3

    obj1 = test_pb2.Test()
    obj1.ParseFromString(bs2)

    obj2 = test_pb.Test()
    obj2.ParseFromString(bs1)

    eq_obj(data, obj1, obj2)

if __name__ == '__main__':
    #_test(data1)
    _test(data2)
